@@ -533,7 +533,7 @@ static int _c2s_client_mio_callback(mio_t m, mio_action_t a, mio_fd_t fd, void *
         case action_CLOSE:
             log_debug(ZONE, "close action on fd %d", fd->fd);
 
-            log_write(sess->c2s->log, LOG_NOTICE, "[%d] [%s, port=%d] disconnect jid=%s, packets: %i, bytes: %d", sess->fd->fd, sess->ip, sess->port, ((sess->resources)?((char*) jid_full(sess->resources->jid)):"unbound"), sess->packet_count, sess->s->rbytes_total);
+            log_write(sess->c2s->log, LOG_NOTICE, "[%d] [%s, port=%d] disconnect jid=%s, packets: %i, bytes: %d", sess->fd->fd, sess->ip, sess->port, ((sess->resources)?((char*) jid_full(sess->resources->jid)):"unbound"), sess->packet_count, sess->s->tbytes);
 
             /* tell the sm to close their session */
             if(sess->active)
@@ -562,7 +562,8 @@ static int _c2s_client_mio_callback(mio_t m, mio_action_t a, mio_fd_t fd, void *
         case action_ACCEPT:
             log_debug(ZONE, "accept action on fd %d", fd->fd);
 
-            getpeername(fd->fd, (struct sockaddr *) &sa, &namelen);
+            if(getpeername(fd->fd, (struct sockaddr *) &sa, &namelen) < 0)
+                return 1;
             port = j_inet_getport(&sa);
 
             log_write(c2s->log, LOG_NOTICE, "[%d] [%s, port=%d] connect", fd->fd, (char *) data, port);
@@ -599,7 +600,8 @@ static int _c2s_client_mio_callback(mio_t m, mio_action_t a, mio_fd_t fd, void *
             sess->s->port = sess->port;
 
             /* find out which port this is */
-            getsockname(fd->fd, (struct sockaddr *) &sa, &namelen);
+            if(getsockname(fd->fd, (struct sockaddr *) &sa, &namelen) < 0)
+                return 1;
             port = j_inet_getport(&sa);
 
             /* remember it */
@@ -968,7 +970,6 @@ int c2s_router_sx_callback(sx_t s, sx_event_t e, void *data, void *arg) {
                     }
 
                     /* build temporary resource to close session for */
-                    tres = NULL;
                     tres = (bres_t) calloc(1, sizeof(struct bres_st));
                     tres->jid = jid_new(NAD_AVAL(nad, target), NAD_AVAL_L(nad, target));
 
@@ -1291,11 +1292,9 @@ int c2s_router_sx_callback(sx_t s, sx_event_t e, void *data, void *arg) {
 
                 /* sm is bouncing something */
                 if(nad_find_attr(nad, 1, ns, "failed", NULL) >= 0) {
-                    if(s) {
-                        /* there's really no graceful way to handle this */
-                        sx_error(s, stream_err_INTERNAL_SERVER_ERROR, "session manager failed control action");
-                        sx_close(s);
-                    }
+                    /* there's really no graceful way to handle this */
+                    sx_error(s, stream_err_INTERNAL_SERVER_ERROR, "session manager failed control action");
+                    sx_close(s);
 
                     nad_free(nad);
                     return 0;
