@@ -22,6 +22,9 @@
 
 #include <stringprep.h>
 #include <unistd.h>
+#ifdef HAVE_SSL
+#include <openssl/rand.h>
+#endif
 
 static sig_atomic_t s2s_shutdown = 0;
 sig_atomic_t s2s_lost_router = 0;
@@ -136,11 +139,6 @@ static void _s2s_config_expand(s2s_t s2s) {
 
     s2s->packet_stats = config_get_one(s2s->config, "stats.packet", 0);
 
-    /*
-     * If no origin IP is specified, use local IP as the originating one:
-     * it makes most sense, at least for SSL'ized connections.
-     * APPLE: make origin an array of addresses so that both IPv4 and IPv6 can be specified.
-     */
     s2s->local_ip = config_get_one(s2s->config, "local.ip", 0);
     if(s2s->local_ip == NULL)
         s2s->local_ip = "0.0.0.0";
@@ -148,23 +146,30 @@ static void _s2s_config_expand(s2s_t s2s) {
         s2s->origin_ips = elem->values;
         s2s->origin_nips = elem->nvalues;
     }
-    if (s2s->origin_nips == 0) {
-        s2s->origin_ips = (const char **)malloc(sizeof(s2s->origin_ips));
-        s2s->origin_ips[0] = strdup(s2s->local_ip);
-        s2s->origin_nips = 1;
-    }
 
     s2s->local_port = j_atoi(config_get_one(s2s->config, "local.port", 0), 0);
 
     if(config_get(s2s->config, "local.secret") != NULL)
         s2s->local_secret = strdup(config_get_one(s2s->config, "local.secret", 0));
     else {
+#ifdef HAVE_SSL
+        if (!RAND_bytes(secret, 40)) {
+            log_write(s2s->log, LOG_ERR, "Failed to pull 40 RAND_bytes");
+            exit(1);
+        }
+#else
+#       warning "Using unsecure random number generator for dialback keys"
+        log_write(s2s->log, LOG_WARNING, "Using unsecure random number generator for dialback keys - set local.secret to a random string!");
+#endif
         for(i = 0; i < 40; i++) {
+#ifdef HAVE_SSL
+            r = (int) (36.0 * secret[i] / 255);
+#else
             r = (int) (36.0 * rand() / RAND_MAX);
+#endif
             secret[i] = (r >= 0 && r <= 9) ? (r + 48) : (r + 87);
         }
         secret[40] = '\0';
-
         s2s->local_secret = strdup(secret);
     }
 
@@ -245,7 +250,7 @@ static void _s2s_hosts_expand(s2s_t s2s)
 
         host->host_verify_mode = j_atoi(j_attr((const char **) elem->attrs[i], "verify-mode"), 0);
 
-		host->host_private_key_password = j_attr((const char **) elem->attrs[i], "private-key-password");
+        host->host_private_key_password = j_attr((const char **) elem->attrs[i], "private-key-password");
 
         host->host_ciphers = j_attr((const char **) elem->attrs[i], "ciphers");
 
@@ -634,7 +639,7 @@ int _s2s_populate_whitelist_domains(s2s_t s2s, const char **values, int nvalues)
     int i, j;
     int elem_len;
     s2s->whitelist_domains = (char **)malloc(sizeof(char*) * (nvalues));
-    memset(s2s->whitelist_domains, 0, (sizeof(char *) * (nvalues)));    
+    memset(s2s->whitelist_domains, 0, (sizeof(char *) * (nvalues)));
     for (i = 0, j = 0; i < nvalues; i++) {
         elem_len = strlen(values[i]);
         if (elem_len > MAX_DOMAIN_LEN) {
@@ -667,7 +672,7 @@ int s2s_domain_in_whitelist(s2s_t s2s, const char *in_domain) {
     int dotcount;
     char **segments = NULL;
     char **dst = NULL;
-    char *seg_tmp = NULL;    
+    char *seg_tmp = NULL;
     int seg_tmp_len;
     char matchstr[MAX_DOMAIN_LEN + 1];
     int domain_index;
@@ -713,7 +718,7 @@ int s2s_domain_in_whitelist(s2s_t s2s, const char *in_domain) {
         if (domain[x] == '.')
             dotcount++;
     }
-        
+
     segments = (char **)malloc(sizeof(char*) * (dotcount + 1));
     if (segments == NULL) {
         log_write(s2s->log, LOG_ERR, "s2s_domain_in_whitelist: malloc() error");
@@ -744,14 +749,12 @@ int s2s_domain_in_whitelist(s2s_t s2s, const char *in_domain) {
         seg_tmp_len = strlen(seg_tmp);
         if (seg_tmp_len > MAX_DOMAIN_LEN) {
             log_write(s2s->log, LOG_NOTICE, "s2s_domain_in_whitelist: domain contains a segment greater than %s chars", MAX_DOMAIN_LEN);
-            if (seg_tmp != NULL) {
-                free(seg_tmp);
-                seg_tmp = NULL;
-            }
+            free(seg_tmp);
+            seg_tmp = NULL;
             for (x = 0; x < segcount; x++) {
                 free(segments[x]);
                 segments[x] = NULL;
-            }   
+            }
             free(segments);
             segments = NULL;
             return 0;
@@ -761,15 +764,13 @@ int s2s_domain_in_whitelist(s2s_t s2s, const char *in_domain) {
         if (*dst != NULL) {
             strncpy(*dst, seg_tmp, seg_tmp_len + 1);
             (*dst)[seg_tmp_len] = '\0';
-        } else { 
-            if (seg_tmp != NULL) {
-                free(seg_tmp);
-                seg_tmp = NULL;
-            }
+        } else {
+            free(seg_tmp);
+            seg_tmp = NULL;
             for (x = 0; x < segcount; x++) {
                 free(segments[x]);
                 segments[x] = NULL;
-            }   
+            }
             free(segments);
             segments = NULL;
             log_write(s2s->log, LOG_ERR, "s2s_domain_in_whitelist: malloc() error");
@@ -797,12 +798,12 @@ int s2s_domain_in_whitelist(s2s_t s2s, const char *in_domain) {
                     for (x = 0; x < segcount; x++) {
                         free(segments[x]);
                         segments[x] = NULL;
-                    }   
+                    }
                     free(segments);
                     segments = NULL;
                     return 1;
-                } 
-                else { 
+                }
+                else {
                     //log_debug(ZONE, "matchstr: %s (len %d) does not match whitelist_domains[%d]: %s (len %d)", &matchstr, strlen((const char *)&matchstr), wl_index, s2s->whitelist_domains[wl_index], strlen(s2s->whitelist_domains[wl_index]));
                 }
             }
@@ -811,11 +812,11 @@ int s2s_domain_in_whitelist(s2s_t s2s, const char *in_domain) {
     for (x = 0; x < segcount; x++) {
         free(segments[x]);
         segments[x] = NULL;
-    }   
+    }
     free(segments);
     segments = NULL;
 
-    return 0;    
+    return 0;
 }
 
 JABBER_MAIN("jabberd2s2s", "Jabber 2 S2S", "Jabber Open Source Server: Server to Server", "jabberd2router\0")
@@ -1072,13 +1073,13 @@ JABBER_MAIN("jabberd2s2s", "Jabber 2 S2S", "Jabber Open Source Server: Server to
 #endif
             if(s2s->packet_stats != NULL) {
                 int fd = open(s2s->packet_stats, O_TRUNC | O_CREAT | O_WRONLY, S_IRUSR | S_IWUSR | S_IRGRP);
-                if(fd) {
+                if(fd >= 0) {
                     char buf[100];
                     int len = snprintf(buf, 100, "%lld\n", s2s->packet_count);
                     write(fd, buf, len);
                     close(fd);
                 } else {
-                    log_write(s2s->log, LOG_ERR, "failed to write packet statistics to: %s", s2s->packet_stats);
+                    log_write(s2s->log, LOG_ERR, "failed to write packet statistics to: %s (%s)", s2s->packet_stats, strerror(errno));
                     s2s_shutdown = 1;
                 }
             }
