@@ -95,7 +95,7 @@ static void _disco_unify_lists(disco_t d) {
 
     if(d->un != NULL)
         xhash_free(d->un);
-    
+
     d->un = xhash_new(101);
 
     /* dynamic overrieds static */
@@ -150,7 +150,7 @@ static pkt_t _disco_info_result(module_t mod, disco_t d) {
     if(xhash_iter_first(mod->mm->sm->features))
         do {
             xhash_iter_get(mod->mm->sm->features, &key, &keylen, NULL);
-            
+
             el = nad_append_elem(pkt->nad, ns, "feature", 3);
             nad_set_attr(pkt->nad, el, -1, "var", (char *) key, keylen);
         } while(xhash_iter_next(mod->mm->sm->features));
@@ -233,7 +233,7 @@ static mod_ret_t _disco_pkt_sm_populate(mod_instance_t mi, pkt_t pkt)
 {
     module_t mod = mi->mod;
     disco_t d = (disco_t) mod->private;
-    int ns, qelem, elem, attr;
+    int ns, query, elem, attr;
     service_t svc;
 
     /* it has to come from the service itself - don't want any old user messing with the table */
@@ -244,9 +244,11 @@ static mod_ret_t _disco_pkt_sm_populate(mod_instance_t mi, pkt_t pkt)
     }
 
     ns = nad_find_scoped_namespace(pkt->nad, uri_DISCO_INFO, NULL);
-    qelem = nad_find_elem(pkt->nad, 1, ns, "query", 1);
-    
-    elem = nad_find_elem(pkt->nad, qelem, ns, "identity", 1);
+    query = nad_find_elem(pkt->nad, 1, ns, "query", 1);
+    if(query < 0)
+        return -stanza_err_BAD_REQUEST;
+
+    elem = nad_find_elem(pkt->nad, query, ns, "identity", 1);
     if(elem < 0)
         return -stanza_err_BAD_REQUEST;
 
@@ -296,7 +298,7 @@ static mod_ret_t _disco_pkt_sm_populate(mod_instance_t mi, pkt_t pkt)
         strcpy(svc->type, "unknown");
 
     /* features */
-    elem = nad_find_elem(pkt->nad, qelem, -1, "feature", 1);
+    elem = nad_find_elem(pkt->nad, query, -1, "feature", 1);
     while(elem >= 0)
     {
         attr = nad_find_attr(pkt->nad, elem, -1, "var", NULL);
@@ -346,7 +348,7 @@ static mod_ret_t _disco_pkt_sm(mod_instance_t mi, pkt_t pkt) {
     disco_t d = (disco_t) mod->private;
     pkt_t result;
     int node, ns;
-    
+
     /* disco info results go to a seperate function */
     if(pkt->type == pkt_IQ_RESULT && pkt->ns == ns_DISCO_INFO)
         return _disco_pkt_sm_populate(mi, pkt);
@@ -590,7 +592,7 @@ static mod_ret_t _disco_pkt_router(mod_instance_t mi, pkt_t pkt)
         _disco_unify_lists(d);
         _disco_generate_packets(mod, d);
     }
-    
+
     /* done */
     pkt_free(pkt);
 
@@ -653,10 +655,10 @@ DLLEXPORT int module_init(mod_instance_t mi, const char *arg)
 
     if(d->agents)
         log_debug(ZONE, "agents compat enabled");
-    
+
     /* our data */
     mod->private = (void *) d;
-    
+
     /* our handlers */
     mod->pkt_sm = _disco_pkt_sm;
     mod->in_sess = _disco_in_sess;
